@@ -5,50 +5,39 @@ use Event ();
 sub io {
    my ($class, %arg) = @_;
    $arg{fd} = delete $arg{fh};
-   my $rcb = \$arg{cb};
    bless \(my $x = Event->io (
       %arg,
-      cb => sub {
-         $$rcb->($_[0]->got . "")
-      },
+      cb => $arg{cb},
    )), $class
 }
 
 sub timer {
    my ($class, %arg) = @_;
-   my $rcb = \$arg{cb};
-   bless \(my $x = Event->timer (
+   my $cb = $arg{cb};
+   bless \(my $w = Event->timer (
       %arg,
       cb => sub {
-         # watcher gets canceled automatically
-         $$rcb->();
+         $_[0]->w->cancel;
+         $cb->();
       },
    )), $class
 }
 
-sub cancel {
-   my ($self) = @_;
-
-   $$self->cancel;
-}
-
 sub DESTROY {
-   my ($self) = @_;
-
-   $self->cancel;
+   ${$_[0]}->cancel;
 }
 
 sub condvar {
    my $class = shift;
 
-   bless \my $x, AnyEvent::Impl::Event::CondVar::
+   bless \my $flag, $class
 }
 
-sub AnyEvent::Impl::Event::CondVar::broadcast {
+sub broadcast {
    ${$_[0]}++;
 }
 
-sub AnyEvent::Impl::Event::CondVar::wait {
+sub wait {
    Event::one_event() while !${$_[0]};
 }
 
