@@ -4,13 +4,6 @@ use Glib ();
 
 my $maincontext = Glib::MainContext->default;
 
-my %RWE = (
-   hup => 'rw',
-   in  => 'r',
-   out => 'w',
-   pri => 'e',
-);
-
 sub io {
    my ($class, %arg) = @_;
    
@@ -18,12 +11,11 @@ sub io {
    my $rcb = \$self->{cb};
 
    # some glibs need hup, others error with it, YMMV
-   push @cond, "in",  "hup" if $self->{poll} =~ /r/i;
-   push @cond, "out", "hup" if $self->{poll} =~ /w/i;
-   push @cond, "pri"        if $self->{poll} =~ /e/i;
+   push @cond, "in",  "hup" if $self->{poll} eq "r";
+   push @cond, "out", "hup" if $self->{poll} eq "w";
 
    $self->{source} = add_watch Glib::IO fileno $self->{fh}, \@cond, sub {
-      $$rcb->(join "", map $RWE{$_}, @{ $_[1] });
+      $$rcb->();
       ! ! $$rcb
    };
 
@@ -44,35 +36,28 @@ sub timer {
    $self
 }
 
-sub cancel {
-   my ($self) = @_;
-
-   remove Glib::Source delete $self->{source} if $self->{source};
-   $self->{cb} = undef;
-   delete $self->{cb};
-}
-
 sub DESTROY {
    my ($self) = @_;
 
-   $self->cancel;
+   remove Glib::Source delete $self->{source} if $self->{source};
+   # need to undef $cb because we hold references to it
+   $self->{cb} = undef;
+   %$self = ();
 }
 
 sub condvar {
    my $class = shift;
 
-   bless \my $x, AnyEvent::Impl::Glib::CondVar::
+   bless \my $flag, $class
 }
 
-sub AnyEvent::Impl::Glib::CondVar::broadcast {
+sub broadcast {
    ${$_[0]}++;
 }
 
-sub AnyEvent::Impl::Glib::CondVar::wait {
+sub wait {
    $maincontext->iteration (1) while !${$_[0]};
 }
 
-$AnyEvent::MODEL = __PACKAGE__;
-
 1
 
